@@ -35,6 +35,7 @@ import           Basement.Numerical.Additive
 import           Basement.Numerical.Subtractive
 import           Basement.Types.OffsetSize
 import           Basement.Types.Char7 (Char7(..))
+import           Basement.IntegralConv
 import           Basement.PrimType
 import           Basement.UTF8.Helper
 import           Basement.UTF8.Table
@@ -94,9 +95,9 @@ nextWith h ba n =
 -- will fail bad if apply at the beginning of string or an empty string.
 prev :: Indexable container Word8 => container -> Offset Word8 -> StepBack
 prev ba offset =
-    case index ba prevOfs1 of
-        (W8# v1) | isContinuation# v1 -> atLeast2 (maskContinuation# v1)
-                 | otherwise          -> StepBack (toChar# v1) prevOfs1
+    case integralUpsize $ index ba prevOfs1 of
+        (W# v1) | isContinuationW# v1 -> atLeast2 (maskContinuation# v1)
+                | otherwise           -> StepBack (toChar# v1) prevOfs1
   where
     sz1 = CountOf 1
     !prevOfs1 = offset `offsetMinusE` sz1
@@ -104,16 +105,16 @@ prev ba offset =
     prevOfs3 = prevOfs2 `offsetMinusE` sz1
     prevOfs4 = prevOfs3 `offsetMinusE` sz1
     atLeast2 !v  =
-        case index ba prevOfs2 of
-            (W8# v2) | isContinuation# v2 -> atLeast3 (or# (uncheckedShiftL# (maskContinuation# v2) 6#) v)
-                     | otherwise          -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader2# v2) 6#) v)) prevOfs2
+        case integralUpsize $ index ba prevOfs2 of
+            (W# v2) | isContinuationW# v2 -> atLeast3 (or# (uncheckedShiftL# (maskContinuation# v2) 6#) v)
+                    | otherwise          -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader2# v2) 6#) v)) prevOfs2
     atLeast3 !v =
-        case index ba prevOfs3 of
-            (W8# v3) | isContinuation# v3 -> atLeast4 (or# (uncheckedShiftL# (maskContinuation# v3) 12#) v)
-                     | otherwise          -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader3# v3) 12#) v)) prevOfs3
+        case integralUpsize $ index ba prevOfs3 of
+            (W# v3) | isContinuationW# v3 -> atLeast4 (or# (uncheckedShiftL# (maskContinuation# v3) 12#) v)
+                    | otherwise           -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader3# v3) 12#) v)) prevOfs3
     atLeast4 !v =
-        case index ba prevOfs4 of
-            (W8# v4) -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader4# v4) 18#) v)) prevOfs4
+        case integralUpsize $ index ba prevOfs4 of
+            (W# v4) -> StepBack (toChar# (or# (uncheckedShiftL# (maskHeader4# v4) 18#) v)) prevOfs4
 
 prevSkip :: Indexable container Word8 => container -> Offset Word8 -> Offset Word8
 prevSkip ba offset = loop (offset `offsetMinusE` sz1)
@@ -139,21 +140,21 @@ writeUTF8 mba !i !c
     !(I# xi) = fromEnum c
     !x       = int2Word# xi
 
-    encode1 = write mba i (W8# x) >> pure (i + Offset 1)
+    encode1 = write mba i (W8# (wordToWord8# x)) >> pure (i + Offset 1)
     encode2 = do
         let x1  = or# (uncheckedShiftRL# x 6#) 0xc0##
             x2  = toContinuation x
-        write mba i     (W8# x1)
-        write mba (i+1) (W8# x2)
+        write mba i     (W8# (wordToWord8# x1))
+        write mba (i+1) (W8# (wordToWord8# x2))
         pure (i + Offset 2)
 
     encode3 = do
         let x1  = or# (uncheckedShiftRL# x 12#) 0xe0##
             x2  = toContinuation (uncheckedShiftRL# x 6#)
             x3  = toContinuation x
-        write mba i            (W8# x1)
-        write mba (i+Offset 1) (W8# x2)
-        write mba (i+Offset 2) (W8# x3)
+        write mba i            (W8# (wordToWord8# x1))
+        write mba (i+Offset 1) (W8# (wordToWord8# x2))
+        write mba (i+Offset 2) (W8# (wordToWord8# x3))
         pure (i + Offset 3)
 
     encode4 = do
@@ -161,10 +162,10 @@ writeUTF8 mba !i !c
             x2  = toContinuation (uncheckedShiftRL# x 12#)
             x3  = toContinuation (uncheckedShiftRL# x 6#)
             x4  = toContinuation x
-        write mba i            (W8# x1)
-        write mba (i+Offset 1) (W8# x2)
-        write mba (i+Offset 2) (W8# x3)
-        write mba (i+Offset 3) (W8# x4)
+        write mba i            (W8# (wordToWord8# x1))
+        write mba (i+Offset 1) (W8# (wordToWord8# x2))
+        write mba (i+Offset 2) (W8# (wordToWord8# x3))
+        write mba (i+Offset 3) (W8# (wordToWord8# x4))
         pure (i + Offset 4)
 
     toContinuation :: Word# -> Word#
