@@ -35,7 +35,8 @@ import           Basement.Compat.Base
 
 -- basic instances
 import           GHC.Types
-import           GHC.Prim
+import           GHC.Prim hiding (word64ToWord#)
+import qualified GHC.Prim
 import           GHC.Int
 import           GHC.Word
 import           Basement.Numerical.Number
@@ -271,11 +272,23 @@ instance (NatWithinBound (CountOf ty) n, KnownNat n, PrimType ty)
     tryFrom = BlockN.toBlockN . UArray.toBlock . BoxArray.mapToUnboxed id
 
 instance (KnownNat n, NatWithinBound Word8 n) => From (Zn64 n) Word8 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . unZn64 where narrow (W64# w) = W8# (wordToWord8# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . unZn64 where narrow (W64# w) = W8# (wordToWord8# (word64ToWord# w))
+#endif
 instance (KnownNat n, NatWithinBound Word16 n) => From (Zn64 n) Word16 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . unZn64 where narrow (W64# w) = W16# (wordToWord16# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . unZn64 where narrow (W64# w) = W16# (wordToWord16# (word64ToWord# w))
+#endif
 instance (KnownNat n, NatWithinBound Word32 n) => From (Zn64 n) Word32 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . unZn64 where narrow (W64# w) = W32# (wordToWord32# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . unZn64 where narrow (W64# w) = W32# (wordToWord32# (word64ToWord# w))
+#endif
 instance From (Zn64 n) Word64 where
     from = unZn64
 instance From (Zn64 n) Word128 where
@@ -284,11 +297,23 @@ instance From (Zn64 n) Word256 where
     from = from . unZn64
 
 instance (KnownNat n, NatWithinBound Word8 n) => From (Zn n) Word8 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W8# (wordToWord8# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W8# (wordToWord8# (word64ToWord# w))
+#endif
 instance (KnownNat n, NatWithinBound Word16 n) => From (Zn n) Word16 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W16# (wordToWord16# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W16# (wordToWord16# (word64ToWord# w))
+#endif
 instance (KnownNat n, NatWithinBound Word32 n) => From (Zn n) Word32 where
+#if __GLASGOW_HASKELL__ >= 904
+    from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W32# (wordToWord32# (word64ToWord# (GHC.Prim.word64ToWord# w)))
+#else
     from = narrow . naturalToWord64 . unZn where narrow (W64# w) = W32# (wordToWord32# (word64ToWord# w))
+#endif
 instance (KnownNat n, NatWithinBound Word64 n) => From (Zn n) Word64 where
     from = naturalToWord64 . unZn
 instance (KnownNat n, NatWithinBound Word128 n) => From (Zn n) Word128 where
