@@ -7,6 +7,7 @@ module Basement.Numerical.Number
 import           Basement.Compat.Base
 import           Basement.Compat.C.Types
 import           Basement.Compat.Natural
+import           Basement.Compat.NumLiteral
 import           Data.Bits
 import qualified Prelude
 
@@ -14,12 +15,12 @@ import qualified Prelude
 --
 -- all number are Enum'erable, meaning that you can move to
 -- next element
-class (Enum a, Eq a, Ord a, Integral a) => IsIntegral a where
+class (Integral a, Eq a, Ord a) => IsIntegral a where
     {-# MINIMAL toInteger #-}
     toInteger :: a -> Integer
 
 -- | Non Negative Number literals, convertible through the generic Natural type
-class (Enum a, Eq a, Ord a, Integral a, IsIntegral a) => IsNatural a where
+class IsIntegral a => IsNatural a where
     {-# MINIMAL toNatural #-}
     toNatural :: a -> Natural
 
